==> ask-cli-1.7.8.tgz: package/lib/clients/git-client.js <==
const { spawnSync, execSync } = require('child_process');
const git = require('simple-git');

module.exports = {
    getGitVersion,
    init,
    configureCredentialHelper,
    addOrigin,
    clone,
    fetchAll,
    checkoutBranch,
    add,
    mergeBranchOneIntoBranchTwo,
    push,
    isBranchStateDirty,
    hasFileChanged,
    isBranchOneAheadOfBranchTwo,
    diffRemote,
    getCurrentBranch
};

const isWindows = process.platform === 'win32';
const quotes = isWindows ? `"` : `'`;

function getGitVersion() {
    const commands = [`git --version`];
    const options = {shell: true, windowsHide: true};
    for (let command of commands) {
        try {
            const output = execSync(command, options);
            return output.toString().trim().split('git version ')[1];
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function init(repoDir, verbosityOptions) {
    const commands = [`git init ${quotes}${repoDir}${quotes}{verbosityOptions.showOutput === false ? ' --quiet' : ''}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function configureCredentialHelper(repoDir, credentialHelperPath, verbosityOptions) {
    const commands = [  `git config --local credential.helper ${quotes}${quotes}`,
                        `git config --local --add credential.helper ${quotes}!${credentialHelperPath}${quotes}`,
                        `git config --local credential.UseHttpPath true`,
                    ];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function addOrigin(repoDir, repoUrl, verbosityOptions) {
    const commands = [`git remote add origin ${repoUrl}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function fetchAll(repoDir, verbosityOptions) {
    const commands = [`git fetch --all${verbosityOptions.showOutput === false ? ' --quiet' : ''}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: false, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function checkoutBranch(repoDir, branch, verbosityOptions) {
    const commands = [`git checkout ${branch}${verbosityOptions.showOutput === false ? ' --quiet' : ''}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function add(repoDir, path, verbosityOptions) {
    const commands = [`git add ${quotes}${path}${quotes}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function mergeBranchOneIntoBranchTwo(repoDir, branchOne, branchTwo, verbosityOptions) {
    const commands = [  `git checkout ${branchTwo}${verbosityOptions.showOutput === false ? ' --quiet' : ''}`,
                        `git merge ${branchOne}`,
                        `git checkout ${branchOne}${verbosityOptions.showOutput === false ? ' --quiet' : ''}`
                    ];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function push(repoDir, branch, verbosityOptions, force) {
    const commands = [`git push origin ${branch}${force ? ' --force' : ''}${verbosityOptions.showOutput === false ? ' --quiet' : ''}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function isBranchStateDirty(repoDir) {
    const commands = [`git status --porcelain`];
    const options = { workingDir: repoDir };
    for (let command of commands) {
        try {
            const output = _execChildProcessSync(command, options);
            return output.toString().trim() ? true : false;
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function hasFileChanged(repoDir, filename) {
    const commands = [`git status --porcelain`];
    const options = { workingDir: repoDir };
    for (let command of commands) {
        try {
            const output = _execChildProcessSync(command, options);
            return output.toString().includes(filename) ? true : false;
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function isBranchOneAheadOfBranchTwo(repoDir, branchOne, branchTwo) {
    const commands = [`git merge-base --is-ancestor ${branchOne} ${branchTwo}`];
    const options = {shell: true, cwd: repoDir, windowsHide: true};
    for (let command of commands) {
        try {
            const execution = spawnSync(command, options);
            if (execution.status !== 0) {
                return true;
            }
            return false;
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function diffRemote(repoDir, remoteBranch, verbosityOptions) {
    const commands = [`git diff origin/${remoteBranch}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            const output = _execChildProcessSync(command, options);
            return output.toString().trim();
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function getCurrentBranch(repoDir) {
    const commands = [`git symbolic-ref --short HEAD`];
    const options = { workingDir: repoDir };
    for (let command of commands) {
        try {
            const output = _execChildProcessSync(command, options);
            return output.toString().trim();
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function clone(cloneUrl, cloneDir, callback) {
    git().silent(true).clone(cloneUrl, cloneDir, (err) => {
        callback(err);
    });
}

/**
 *
 * @param command
 * @param options { showStdOut, showStdErr, showCmd, workingDir }
 * @private
 */
function _execChildProcessSync(command, options) {
    const { showStdOut, showStdErr, showCmd, workingDir } = options;
    const execOptions = {
        stdio: [null, showStdOut ? 1:null, showStdErr ? 2:null],
        shell: true,
        windowsHide: true
    };
    if (workingDir) {
        execOptions.cwd = options.workingDir;
    }
    if (showCmd) {
        console.log(command);
    }
    return execSync(command, execOptions);
}

==> ask-cli-1.7.12.tgz: package/lib/clients/git-client.js <==
const { spawnSync, execSync } = require('child_process');
const git = require('simple-git');

module.exports = {
    getGitVersion,
    init,
    configureCredentialHelper,
    addOrigin,
    clone,
    fetchAll,
    checkoutBranch,
    add,
    mergeBranchOneIntoBranchTwo,
    push,
    isBranchStateDirty,
    hasFileChanged,
    isBranchOneAheadOfBranchTwo,
    diffRemote,
    getCurrentBranch
};

const isWindows = process.platform === 'win32';
const quotes = isWindows ? `"` : `'`;

function getGitVersion() {
    const commands = [`git --version`];
    const options = {shell: true, windowsHide: true};
    for (let command of commands) {
        try {
            const output = execSync(command, options);
            return output.toString().trim().split('git version ')[1];
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function init(repoDir, verbosityOptions) {
    const commands = [`git init ${quotes}${repoDir}${quotes}${verbosityOptions.showOutput === false ? ' --quiet' : ''}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function configureCredentialHelper(repoDir, credentialHelperPath, verbosityOptions) {
    const commands = [  `git config --local credential.helper ${quotes}${quotes}`,
                        `git config --local --add credential.helper ${quotes}!${credentialHelperPath}${quotes}`,
                        `git config --local credential.UseHttpPath true`,
                    ];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function addOrigin(repoDir, repoUrl, verbosityOptions) {
    const commands = [`git remote add origin ${repoUrl}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function fetchAll(repoDir, verbosityOptions) {
    const commands = [`git fetch --all${verbosityOptions.showOutput === false ? ' --quiet' : ''}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: false, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function checkoutBranch(repoDir, branch, verbosityOptions) {
    const commands = [`git checkout ${branch}${verbosityOptions.showOutput === false ? ' --quiet' : ''}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function add(repoDir, path, verbosityOptions) {
    const commands = [`git add ${quotes}${path}${quotes}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function mergeBranchOneIntoBranchTwo(repoDir, branchOne, branchTwo, verbosityOptions) {
    const commands = [  `git checkout ${branchTwo}${verbosityOptions.showOutput === false ? ' --quiet' : ''}`,
                        `git merge ${branchOne}`,
                        `git checkout ${branchOne}${verbosityOptions.showOutput === false ? ' --quiet' : ''}`
                    ];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function push(repoDir, branch, verbosityOptions, force) {
    const commands = [`git push origin ${branch}${force ? ' --force' : ''}${verbosityOptions.showOutput === false ? ' --quiet' : ''}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            _execChildProcessSync(command, options);
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function isBranchStateDirty(repoDir) {
    const commands = [`git status --porcelain`];
    const options = { workingDir: repoDir };
    for (let command of commands) {
        try {
            const output = _execChildProcessSync(command, options);
            return output.toString().trim() ? true : false;
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function hasFileChanged(repoDir, filename) {
    const commands = [`git status --porcelain`];
    const options = { workingDir: repoDir };
    for (let command of commands) {
        try {
            const output = _execChildProcessSync(command, options);
            return output.toString().includes(filename) ? true : false;
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function isBranchOneAheadOfBranchTwo(repoDir, branchOne, branchTwo) {
    const commands = [`git merge-base --is-ancestor ${branchOne} ${branchTwo}`];
    const options = {shell: true, cwd: repoDir, windowsHide: true};
    for (let command of commands) {
        try {
            const execution = spawnSync(command, options);
            if (execution.status !== 0) {
                return true;
            }
            return false;
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function diffRemote(repoDir, remoteBranch, verbosityOptions) {
    const commands = [`git diff origin/${remoteBranch}`];
    const options = { showStdOut: verbosityOptions.showOutput, showStdErr: true, showCmd: verbosityOptions.showCommand, workingDir: repoDir };
    for (let command of commands) {
        try {
            const output = _execChildProcessSync(command, options);
            return output.toString().trim();
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function getCurrentBranch(repoDir) {
    const commands = [`git symbolic-ref --short HEAD`];
    const options = { workingDir: repoDir };
    for (let command of commands) {
        try {
            const output = _execChildProcessSync(command, options);
            return output.toString().trim();
        } catch (ex) {
            throw(`${command} Failed.`);
        }
    }
}

function clone(cloneUrl, cloneDir, callback) {
    git().silent(true).clone(cloneUrl, cloneDir, (err) => {
        callback(err);
    });
}

/**
 *
 * @param command
 * @param options { showStdOut, showStdErr, showCmd, workingDir }
 * @private
 */
function _execChildProcessSync(command, options) {
    const { showStdOut, showStdErr, showCmd, workingDir } = options;
    const execOptions = {
        stdio: [null, showStdOut ? 1:null, showStdErr ? 2:null],
        shell: true,
        windowsHide: true
    };
    if (workingDir) {
        execOptions.cwd = options.workingDir;
    }
    if (showCmd) {
        console.log(command);
    }
    return execSync(command, execOptions);
}
